__version__ = '1.2.3'
from .meteorology import Meteorology
from .create_amy_epw_files_for_years_and_wmos import create_amy_epw_files_for_years_and_wmos
from .analyze_noaa_isd_lite_files import analyze_noaa_isd_lite_files
from .analyze_noaa_isd_lite_file import analyze_noaa_isd_lite_file
from .get_wmo_station_location import get_wmo_station_location
from .get_noaa_isd_lite_file import get_noaa_isd_lite_file
from .get_tmy_epw_file import get_tmy_epw_file
from .create_amy_epw_file import create_amy_epw_file
from .exceptions import DownloadNotAllowedError



import click
import diyepw
import os
import pandas as pd


@click.command()
@click.option(
    '--max-records-to-interpolate',
    default=6,
    show_default=True,
    type=int,
    help="""The maximum number of consecutive records to interpolate. See the documentation of the
            pandas.DataFrame.interpolate() method's "limit" argument for more details. Basically,
            if a sequence of fields up to the length defined by this argument are missing, those 
            missing values will be interpolated linearly using the values of the fields immediately 
            preceding and following the missing field(s). If a sequence of fields is longer than this
            limit, then those fields' values will be imputed instead (see --max-records-to-impute)
            """
)
@click.option(
    '--max-records-to-impute',
    default=48,
    show_default=True,
    type=int,
    help=f"""The maximum number of records to impute. For groups of missing records larger than the
            limit set by --max-records-to-interpolate but up to --max-records-to-impute, we replace the 
            missing values using the average of the value two weeks prior and the value two weeks after 
            the missing value. If there are more consecutive missing records than this limit, then the 
            file will not be processed, and will be added to the error file."""
)
@click.option(
    '-o', '--output-path',
    default='.',
    type=click.Path(
        file_okay=False,
        dir_okay=True,
        writable=True,
        resolve_path=True,
    ),
    help="""The path to which output and error files should be written."""
)
@click.argument(
    'path_to_station_list',
    default='./files_to_convert.csv',
    type=click.Path(
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
    ),
)
def create_amy_epw_files(
    max_records_to_interpolate,
    max_records_to_impute,
    output_path,
    path_to_station_list,
):
    """Generate epw files based on the PATH_TO_STATION_LIST as generated by analyze_noaa_data.py, which must be called
       prior to this script. The generated files will be written to the designated --output-path. A list of any files
       that could not be generated due to validation or other errors will be written to errors.csv."""

    # Set path to outputs produced by this script.
    if not os.path.exists(output_path):
        os.mkdir(output_path)

    # Set path to the files where errors should be written
    errors_path = os.path.join(output_path, 'errors.csv')

    # Ensure that the errors file is truncated
    with open(errors_path, 'w'):
        pass

    # Read in list of AMY files that should be used to create EPW files.
    amy_file_list = pd.read_csv(path_to_station_list)
    amy_file_list = amy_file_list[amy_file_list.columns[0]]

    # Initialize the df to hold paths of AMY files that could not be converted to an EPW.
    errors = pd.DataFrame(columns=['file', 'error'])

    num_files = len(amy_file_list)
    for idx, amy_file_path in enumerate(amy_file_list, start=1):
        # The NOAA ISD Lite AMY files are stored in directories named the same as the year they describe, so we
        # use that directory name to get the year
        amy_file_dir = os.path.dirname(amy_file_path)
        year = int(amy_file_dir.split(os.path.sep)[-1])
        next_year = year + 1

        # To get the WMO, we have to parse it out of the filename: it's the portion prior to the first hyphen
        wmo_index = int(os.path.basename(amy_file_path).split('-')[0])

        # Our NOAA ISD Lite input files are organized under inputs/NOAA_ISD_Lite_Raw/ in directories named after their
        # years, and the files are named identically (<WMO>_<###>_<Year>.gz), so we can get the path to the subsequent
        # year's file by switching directories and swapping the year in the file name.
        s = os.path.sep
        amy_subsequent_year_file_path = amy_file_path.replace(s + str(year) + s, s + str(next_year) + s)\
                                                     .replace(f'-{year}.gz', f'-{next_year}.gz')
        try:
            amy_epw_file_path = diyepw.create_amy_epw_file(
                wmo_index=wmo_index,
                year=year,
                max_records_to_impute=max_records_to_impute,
                max_records_to_interpolate=max_records_to_interpolate,
                amy_epw_dir=output_path,
                amy_files=(amy_file_path, amy_subsequent_year_file_path),
                allow_downloads=True,
            )

            click.echo(f"Success! {os.path.basename(amy_file_path)} => {os.path.basename(amy_epw_file_path)} ({idx} / {num_files})")
        except Exception as e:
            errors = errors.append({"file": amy_file_path, "error": str(e)}, ignore_index=True)
            click.echo(f"\n*** Error! {amy_file_path} could not be processed, see {errors_path} for details ({idx} / {num_files})\n")

    click.echo("\nDone!")

    if not errors.empty:
        click.echo(f"{len(errors)} files encountered errors - see {errors_path} for more information")
        errors.to_csv(errors_path, mode='w', index=False)

    click.echo(f"{num_files - len(errors)} files successfully processed. EPWs were written to {output_path}.")



import pandas as pd


def analyze_noaa_isd_lite_file(
        file: str,
        compression: str = 'infer'
):
    """
    Performs an analysis of a single NOAA ISD Lite file, determining whether it is suitable for conversion into an AMY
    EPW file.

    :param file: The path of the NOAA ISD Lite file to be analyzed. The file may be uncompressed, or in any
       compression format permitted by pandas.read_csv(). If it is compressed, it must have a corresponding file
       extension, or the compression parameter must be passed. File name must begin with the WMO index of the
       weather station at which the observations in the file were recorded.
    :param compression: If you pass compressed files that don't end in the typical extension for their
       compression type (e.g. ".gz" for GZIP, ".zip" for ZIP, etc.) then you must pass this to indicate
       what the files' compression format is. See the documentation of the `compression` parameter of
       pandas.read_csv() for more information.

    :return: dict in the form:
        {
          'file': <str> - absolute path to the file,
          'total_rows_missing': <int> - The number of total rows that are missing data in this file,
          'max_consec_rows_missing': <int> - The largest number of consecutive rows that are missing data in this file
        }
    """
    # Read the file into a Pandas dataframe.
    df = pd.read_csv(file,
                     delim_whitespace=True,
                     header=None,
                     compression=compression,
                     names=["Year", "Month", "Day", "Hour", "Air_Temperature",
                            "Dew_Point_Temperature", "Sea_Level_Pressure", "Wind_Direction",
                            "Wind_Speed_Rate", "Sky_Condition_Total_Coverage_Code",
                            "Liquid_Precipitation_Depth_Dimension_1H", "Liquid_Precipitation_Depth_Dimension_6H"]
                     )

    # Take year-month-day-hour columns and convert to datetime stamps.
    df['obs_timestamps'] = pd.to_datetime(pd.DataFrame({'year': df['Year'],
                                                        'month': df['Month'],
                                                        'day': df['Day'],
                                                        'hour': df['Hour']}))

    # Remove unnecessary year, month, day, hour columns
    df = df.drop(columns=['Year', 'Month', 'Day', 'Hour'])

    rows_present = df.shape[0]
    file_description = {
        'file': file,
        'total_rows_missing': 8760 - rows_present,
        'max_consec_rows_missing': 0
    }
    # Skip the work of counting consecutive missing rows for files that have no missing rows
    if file_description['total_rows_missing'] > 0:
        file_description['max_consec_rows_missing'] = _get_max_missing_rows_from_hourly_dataframe(df, 'obs_timestamps')

    return file_description


def _get_max_missing_rows_from_hourly_dataframe(df: pd.DataFrame, timestamp_col_name: str) -> int:
    """
    Given a DataFrame containing hourly timestamps over a year, determine the longest sequence of timestamps
    missing from that DataFrame.

    :param df: The DataFrame from which to retrieve the maximum number of missing timestamps.
    :param timestamp_col_name: The name of the column containing timestamps in the DataFrame `df`
    :return: The size of the longest sequence of missing timestamps from the Dataframe `df`
    """
    # Create series of continuous timestamp values for that year
    all_timestamps = pd.date_range(df[timestamp_col_name].iloc[0], periods=8760, freq='H')

    # Merge to one dataframe containing all continuous timestamp values.
    all_times = pd.DataFrame(all_timestamps, columns=['all_timestamps'])
    df_all_times = pd.merge(all_times, df, how='left', left_on='all_timestamps', right_on=timestamp_col_name)

    # Create series of only the missing timestamp values
    missing_times = df_all_times[df_all_times.isnull().any(axis=1)]
    missing_times = missing_times['all_timestamps']

    if missing_times.empty: # pragma: no cover - This check is good defensive programming, but in practice we don't
        return 0            # actually call this function at all if the number of missing rows is 0

    # Create a series containing the time step distance from the previous timestamp for the missing timestamp values
    missing_times_diff = missing_times.diff()

    # Count the maximum number of consecutive missing time steps.
    counter = 1
    max_missing_rows = 1
    for step in missing_times_diff:
        if step == pd.Timedelta('1h'):
            counter += 1
            if counter > max_missing_rows:
                max_missing_rows = counter
        elif step > pd.Timedelta('1h'):
            counter = 1

    return max_missing_rows




import logging
import os
import sys
import pkg_resources
from datetime import datetime

_LOG_LEVEL = logging.INFO

log_dir = pkg_resources.resource_filename("diyepw", "log")
if not os.path.exists(log_dir): # pragma: no cover
    os.mkdir(log_dir)

_log_path = os.path.join(log_dir, str(datetime.now()).replace(' ', '_').replace(':', '_') +'.log')
_file_handler = logging.FileHandler(_log_path)
_file_handler.setFormatter(logging.Formatter("%(asctime)s [diyepw.%(levelname)s] %(message)s"))
_file_handler.setLevel(_LOG_LEVEL)

_stdout_handler = logging.StreamHandler(sys.stdout)
_stdout_handler.setFormatter(logging.Formatter("%(asctime)s %(message)s"))
_stdout_handler.setLevel(_LOG_LEVEL)

_logger = logging.Logger('diyepw')
_logger.addHandler(_stdout_handler)
_logger.addHandler(_file_handler)



import os
from typing import Iterable
from ._logging import _logger
from .analyze_noaa_isd_lite_file import analyze_noaa_isd_lite_file


def analyze_noaa_isd_lite_files(
        files: Iterable,
        *,
        max_missing_rows: int = 700,
        max_consecutive_missing_rows: int = 48,
        compression: str = 'infer'
):
    """
    Performs an analysis of a set of NOAA ISD Lite files, determining which of them are suitable
    for conversion into AMY EPW files.

    :param files: An collection of paths to the NOAA ISD Lite files to be analyzed. See this package's
        analyze_noaa_isd_lite_file() function for more information.
    :param max_missing_rows: The maximum number of total missing rows to allow in a file.
    :param max_consecutive_missing_rows: The maximum number of consecutive missing rows to allow in a file.
    :param compression: If you pass compressed files that don't end in the typical extension for their
       compression type (e.g. ".gz" for GZIP, ".zip" for ZIP, etc.) then you must pass this to indicate
       what the files' compression format is. See the documentation of the `compression` parameter of
       pandas.read_csv() for more information.

    :return: dict in the form:
        {
          "good": [<file_description>, ...],
          "too_many_total_rows_missing": [<file_description>, ...],
          "too_many_consecutive_rows_missing": [<file_description>, ...]
        }
        ...where <file_description> is itself a dict in the form:
        {
          'file': <str> - absolute path to the file,
          'total_rows_missing': <int> - The number of total rows that are missing data in this file,
          'max_consec_rows_missing': <int> - The largest number of consecutive rows that are missing data in this file
        }
    """

    too_many_missing_rows = []
    too_many_consecutive_missing_rows = []
    good_files = []

    for file in files:

        # It's nicer to work with absolute paths, especially since we are going to put this path in a
        # file to share with another script - otherwise that other script needs to know where this
        # script is located to make sense of the relative paths
        file = os.path.abspath(file)

        file_description = analyze_noaa_isd_lite_file(file, compression)

        # Depending on how many total and consecutive rows are missing, add the current file to one of our
        # collections to be returned to the caller
        if file_description['total_rows_missing'] > max_missing_rows:
            _logger.info(f"{file} has too many total missing rows")
            too_many_missing_rows.append(file_description)
        elif file_description['max_consec_rows_missing'] > max_consecutive_missing_rows:
            _logger.info(f"{file} has too many consecutive missing rows")
            too_many_consecutive_missing_rows.append(file_description)
        else:
            _logger.info(f"{file} looks good!")
            good_files.append(file_description)

    return {
        "good": good_files,
        "too_many_total_rows_missing": too_many_missing_rows,
        "too_many_consecutive_rows_missing": too_many_consecutive_missing_rows
    }




import unittest
import diyepw
import tempfile
import pvlib
import pandas as pd


class TmyCreateAmyEpwFilesForYearsAndWmosTest(unittest.TestCase):
    """
    Tests of the behavior of the create_amy_epw_files_for_years_and_wmos() function
    """

    def test_file_creation_with_download(self):
        """Verify the standard use case - files that are not already available being requested with
           downloads enabled"""
        years = [2015, 2016]
        wmo_indices = [725300, 725957]

        with tempfile.TemporaryDirectory() as output_dir:
            # Loop twice so that we exercise the buffering behavior of this function as well - the second time through,
            # the requested file will already exist due to the first call, and a shortcut skipping downloading should
            # be invoked. It's difficult to verify that this occurs from the caller's perspective, but it can be verified
            # in code coverage.
            for _ in range(2):
                result = diyepw.create_amy_epw_files_for_years_and_wmos(
                    years=years,
                    wmo_indices=wmo_indices,
                    max_records_to_interpolate=2,
                    max_records_to_impute=50,
                    max_missing_amy_rows=10,
                    allow_downloads=True,
                    amy_epw_dir=output_dir
                )
                self.assertEqual(list(result.keys()), years)

                for year in result:
                    self.assertEqual(list(result[year].keys()), wmo_indices)

                    for wmo_index in result[year]:
                        for file_path in result[year][wmo_index]:
                            self._validate_epw_file(file_path)

    def _validate_epw_file(self, file_path):
        """Validate that a given file is a valid EPW file"""

        # Verify that the generated file can be parsed as an EPW
        with open(file_path, "r") as m:
            try:
                parsed_epw, col_names = pvlib.iotools.parse_epw(m)
            except Exception as e:
                raise Exception(f"Error parsing the result of create_amy_epw_file() as an EPW file: {e}")

            # Make sure that parse_epw() actually succeeded in creating a DataFrame instance
            self.assertIsInstance(parsed_epw, pd.DataFrame)




if __name__ == '__main__': # pragma: no cover
    unittest.main()



import unittest
import diyepw
import tempfile
import pvlib
import pandas as pd
import os

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


class TmyCreateAmyEpwFileTest(unittest.TestCase):
    """
    Tests of the behavior of the create_amy_epw_file function
    """

    def test_file_creation_with_download(self):
        """Verify the standard use case - a new file that is not already available being requested with
           downloads enabled"""

        # Loop twice so that we exercise the buffering behavior of this function as well - the second time through,
        # the requested file will already exist due to the first call, and a shortcut skipping downloading should
        # be invoked. It's difficult to verify that this occurs from the caller's perspective, but it can be verified
        # in code coverage.
        for _ in range(2):
            file_path = diyepw.create_amy_epw_file(
                702910,
                2018,
                max_records_to_interpolate=5,
                max_records_to_impute=40,
                allow_downloads=True
            )
            self._validate_epw_file(file_path)

    def test_file_creation_with_amy_files(self):
        """Test the use case that the caller specifies AMY files to be used instead of allowing these
           to be determined automatically"""
        file_path = diyepw.create_amy_epw_file(
            725300,
            2017,
            max_records_to_interpolate=2, # Intentionally very low so that imputation code gets exercised
            max_records_to_impute=20,
            allow_downloads=True,
            amy_files = (
                os.path.join(THIS_DIR, 'files', 'noaa_isd_lite', '725300-2017.gz'),
                os.path.join(THIS_DIR, 'files', 'noaa_isd_lite', '725300-2018.gz')
            )
        )
        self._validate_epw_file(file_path)

    def test_leap_year(self):
        """Test that an AMY EPW file can be generated for a leap year, which is an issue because the
           TMY files we use have the number of hours for a non-leap year, which causes size mismatches
           when mapping AMY data onto the TMY EPW file without special handling"""
        file_path = diyepw.create_amy_epw_file(
            725300,
            2016,
            max_records_to_interpolate=2,
            max_records_to_impute=20,
            allow_downloads=True,
            amy_files = (
                os.path.join(THIS_DIR, 'files', 'noaa_isd_lite', '725300-2016.gz'),
                os.path.join(THIS_DIR, 'files', 'noaa_isd_lite', '725300-2017.gz')
            )
        )
        self._validate_epw_file(file_path)

    def test_validation_errors(self):
        """Verify that invalid inputs result in errors as expected"""

        # It should not be possible to pass both amy_dir and amy_files, since both define which AMY files should
        # be used to handle the call.
        with self.assertRaises(Exception):
            diyepw.create_amy_epw_file(
                724940,
                2018,
                max_records_to_interpolate=5,
                max_records_to_impute=20,
                amy_dir="some_dir",
                amy_files=("some_file", "some_other_file")
            )

        # If amy_files is used to define the current and next year AMY input files, then
        # an Exception should be raised if either of the two is not a real file. We create
        # a temporary file so that one of the paths does exist and pass a dummy for the other.
        with tempfile.NamedTemporaryFile('r') as tmp_file:
            with self.assertRaises(Exception):
                diyepw.create_amy_epw_file(
                    724940,
                    2018,
                    max_records_to_interpolate=5,
                    max_records_to_impute=20,
                    amy_files=(tmp_file.name, "some_other_file")
                )
            with self.assertRaises(Exception):
                diyepw.create_amy_epw_file(
                    724940,
                    2018,
                    max_records_to_interpolate=5,
                    max_records_to_impute=20,
                    amy_files=("some_file", tmp_file.name)
                )

        # An otherwise valid call, passed inputs into which we have injected values that would result in an invalid EPW
        # file being generated, should result in an exception
        with self.assertRaises(Exception):
            with tempfile.TemporaryDirectory() as tmp_dir:
                diyepw.create_amy_epw_file(
                    725300,
                    2017,
                    max_records_to_interpolate=5,
                    max_records_to_impute=20,
                    amy_epw_dir=tmp_dir,
                    allow_downloads=True,
                    amy_files = (
                        os.path.join(THIS_DIR, 'files', 'noaa_isd_lite', '725300-2017_epw_violations.gz'),
                        os.path.join(THIS_DIR, 'files', 'noaa_isd_lite', '725300-2018.gz')
                    )
                )

        # We should get an exception if we specify maximum numbers of missing or consecutive missing AMY input rows
        # that are less than the number of missing rows in our AMY input files
        with self.assertRaises(Exception):
            with tempfile.TemporaryDirectory() as tmp_dir:
                diyepw.create_amy_epw_file(
                    725300,
                    2017,
                    max_records_to_interpolate=5,
                    max_records_to_impute=20,
                    amy_epw_dir=tmp_dir,
                    max_missing_amy_rows=15,
                    amy_files = (
                        os.path.join(THIS_DIR, 'files', 'noaa_isd_lite', 'only_one_row'),
                        os.path.join(THIS_DIR, 'files', 'noaa_isd_lite', '725300-2018.gz')
                    )
                )

    def _validate_epw_file(self, file_path):
        """Validate that a given file is a valid EPW file"""

        # Verify that the generated file can be parsed as an EPW
        with open(file_path, "r") as m:
            try:
                parsed_epw, col_names = pvlib.iotools.parse_epw(m)
            except Exception as e:
                raise Exception(f"Error parsing the result of create_amy_epw_file() as an EPW file: {e}")

            # Make sure that parse_epw() actually succeeded in creating a DataFrame instance
            self.assertIsInstance(parsed_epw, pd.DataFrame)


if __name__ == '__main__': # pragma: no cover
    unittest.main()



import click
import datetime
import diyepw
import os
from typing import List


@click.command()
@click.option(
    '-y', '--years',
    type=str,
    prompt='Which years? (i.e. 2000,2003,2006 or 2000-2005)',
    help="""The years for which to generate AMY EPW files. This is a comma-separated list that can
            include individual years (--years=2000,2003,2006), a range (--years=2000-2005), or
            a combination of both (--years=2000,2003-2005,2007)"""
)
@click.option(
    '-w', '--wmo-indices',
    type=str,
    prompt='Which WMOs? (i.e. 724940,724300)',
    help="""The WMO indices (weather station IDs) for which to generate AMY EPW files. This is a 
            comma-separated list (--wmo-indices=724940,724300). Note that currently only WMO
            indices beginning with 7 (North America) are supported.""",
)
@click.option(
    '--max-records-to-interpolate',
    default=6,
    type=int,
    show_default=True,
    help="""The maximum number of consecutive records to interpolate. See the documentation of the
            pandas.DataFrame.interpolate() method's "limit" argument for more details. Basically,
            if a sequence of fields up to the length defined by this argument are missing, those 
            missing values will be interpolated linearly using the values of the fields immediately 
            preceding and following the missing field(s). If a sequence of fields is longer than this
            limit, then those fields' values will be imputed instead (see --max-records-to-impute)
            """
)
@click.option(
    '--max-records-to-impute',
    default=48,
    show_default=True,
    type=int,
    help="""The maximum number of records to impute. For groups of missing records larger than the
            limit set by --max-records-to-interpolate but up to --max-records-to-impute, we replace the 
            missing values using the average of the value two weeks prior and the value two weeks after 
            the missing value. If there are more consecutive missing records than this limit, then the 
            file will not be processed, and will be added to the error file."""
)
@click.option(
    '--max-missing-amy-rows',
    default=700,
    show_default=True,
    type=int,
    help="""The AMY files corresponding to each requested WMO/year combination will be checked against
         this maximum - any file that is missing more than this number of total observations with
         more than this number of total missing rows will not be generated. Instead, an entry will
         be added to the error file."""
)
@click.option(
    '-o', '--output-path',
    default='.',
    type=click.Path(
        file_okay=False,
        dir_okay=True,
        writable=True,
        resolve_path=True,
    ),
    help="""The path to which output and error files should be written."""
)
def create_amy_epw_files_for_years_and_wmos(
    years,
    wmo_indices,
    max_records_to_interpolate,
    max_records_to_impute,
    max_missing_amy_rows,
    output_path,
):
    """Generate AMY EPW files for a set of years and WMO indices. The generated files will be written to
       the designated --output-path. A list of any files that could not be generated due to validation or other errors
       will be written to epw_validation_errors.csv and errors.csv."""

    # Set path to the directory we'll write created AMY EPW files to.
    if not os.path.exists(output_path):
        os.mkdir(output_path)

    # Set path to the files where errors should be written
    epw_file_violations_path = os.path.join(output_path, 'epw_validation_errors.csv')
    errors_path = os.path.join(output_path, 'errors.csv')

    diyepw.create_amy_epw_files_for_years_and_wmos(
        years=get_years_list(years),
        wmo_indices=get_wmo_indices_list(wmo_indices),
        max_records_to_impute=max_records_to_impute,
        max_records_to_interpolate=max_records_to_interpolate,
        max_missing_amy_rows=max_missing_amy_rows,
        amy_epw_dir=output_path,
        allow_downloads=True,
    )


def get_years_list(years_str: str) -> List[int]:
    """
    Transform the years argument string, which can be formatted like "2000, 2001, 2005-2010" (individual years or
    ranges, comma separated, not necessarily sorted, with optional spaces), into a sorted list of integers
    """
    # Transform the years argument from a string like  to a sorted list
    years_list = []
    years_str = years_str.replace(" ", "")  # Ignore any spaces
    for year_arg_part in years_str.split(","):  # We'll process each comma-separated entry in the list of years
        if "-" in year_arg_part:  # If there is a hyphen, then it's a range like "2000-2010"
            start_year, end_year = year_arg_part.split("-")
            years_list += range(int(start_year), int(end_year) + 1)
        else:  # If there is no hyphen, it's just a single year
            years_list.append(int(year_arg_part))
    years_list.sort()

    # Validate that the years are between 1900 and the present
    this_year = datetime.datetime.now().year
    if min(years_list) < 1900 or max(years_list) > this_year:
        raise Exception(f"Years must be in the range 1900-{this_year}")

    return years_list


def get_wmo_indices_list(wmo_indices_str: str) -> List[int]:
    """
    Transforms the wmo-indices argument, which should be a comma-separated list of WMO indices, into a sorted list of
    integers
    :param wmo_indices_str:
    :return:
    """
    wmo_indices_str = wmo_indices_str.replace(" ", "") # Ignore any spaces

    # Split on "," and convert each value to an integer
    wmo_indices_list = [int(wmo_index) for wmo_index in wmo_indices_str.split(",")]

    return wmo_indices_list



import unittest
import diyepw
import pkg_resources
import tempfile
import os


class TmyGetEpwFileTest(unittest.TestCase):
    """
    Tests of the behavior of the get_tmy_epw_file function
    """

    def test_file_download(self):
        """
        Confirm the function works correctly if all files (the catalog and the file itself) must be downloaded.
        """
        with tempfile.TemporaryDirectory() as output_dir:
            wmo_index = 722016

            # If we don't pass the allow_downloads flag to give explicit permission, the function should recognize that
            # the requested file is not present in output_dir (which is empty) and raise an Exception rather than proceed
            with self.assertRaises(Exception):
                diyepw.get_tmy_epw_file(wmo_index, output_dir=output_dir)

            # Now pass the allow_downloads flag and check that the download behavior works correctly
            # We parse the downloaded file into a Meteorology instance. Just the fact that this succeeds without
            # error is a good indication that the download functioned correctly; we just assert that the class
            # instantiated successfully and that it parsed the expected WMO Index out of the downloaded file
            # to ensure that it seems to have worked as expected
            file_path = diyepw.get_tmy_epw_file(wmo_index, output_dir=output_dir, allow_downloads=True)
            meteorology = diyepw.Meteorology.from_tmy3_file(file_path)
            self.assertIsInstance(meteorology, diyepw.Meteorology)
            self.assertEqual(meteorology.station_number, wmo_index)

            # Call the same method again to exercise the shortcut in the code that will skip the download if
            # the file is already present in output_dir. This is difficult to confirm (unless we wanted to verify
            # that the second call ran faster?) but can be verified by seeing in the coverage report that the
            # code branch associated with that behavior has been invoked.
            diyepw.get_tmy_epw_file(wmo_index, output_dir=output_dir, allow_downloads=True)

        # We need a new temp directory because we need to trigger another download attempt
        with tempfile.TemporaryDirectory() as output_dir:
            # Delete the catalog data file, so that the next call to get_tmy_epw_file() will be forced to trigger
            # the file to be downloaded anew
            os.unlink(pkg_resources.resource_filename("diyepw", "data/tmy_epw_catalogs/tmy_epw_catalog.csv"))

            # With the catalog missing and a fresh temporary output directory, we get the same behavior as above
            # when trying to get a TMY EPW file without passing allow_downloads, but in this case it is due to
            # the catalog failing to download.
            with self.assertRaises(Exception):
                diyepw.get_tmy_epw_file(wmo_index, output_dir=output_dir)

            # Now call it with allow_downloads to ensure that the catalog downloads correctly
            file_path = diyepw.get_tmy_epw_file(wmo_index, output_dir=output_dir, allow_downloads=True)
            meteorology = diyepw.Meteorology.from_tmy3_file(file_path)
            self.assertIsInstance(meteorology, diyepw.Meteorology)
            self.assertEqual(meteorology.station_number, wmo_index)

    def test_argument_validation(self):
        """
        Ensure that the get_tmy_epw_file() method correctly handles invalid arguments
        """

        invalid_args = [
            { "wmo_index": "banana" }, # Invalid WMO index
            { "wmo_index": "777777" }, # Nonexistent WMO index
            { "wmo_index": "722016", 'output_dir': '08jtounaksnfehurousoejfowijf' } # Nonexistent output directory
        ]

        for args in invalid_args:
            with self.assertRaises(Exception):
                diyepw.get_tmy_epw_file(**args)

if __name__ == '__main__': # pragma: no cover
    unittest.main()



"""
Setup file for DIYEPW
:author:   Amanda Smith
:email:    amanda.d.smith@pnnl.gov
License:  BSD 2-Clause, see LICENSE and DISCLAIMER files
"""

import setuptools  # pragma: no coverU


def readme():
    with open('README.md') as f:
        return f.read()


setuptools.setup(  # pragma: no cover
      name='diyepw',
      description='A package offering support for generating EPW (EnergyPlus Weather) files',
      long_description=readme(),
      long_description_content_type="text/markdown",
      url='https://github.com/IMMM-SFA/diyepw',
      author='Amanda Smith',
      author_email='amanda.d.smith@pnnl.gov',
      packages=setuptools.find_packages(),
      package_data={'diyepw': ['data/**', 'test/files/**', 'data/**/*', 'test/files/**/*']},
      license='BSD 2-Clause',
      python_requires='~=3.7',
      install_requires=[
          'click~=8.0.1',
          'numpy~=1.21.2',
          'pandas~=1.3.2',
          'pvlib~=0.8.1',
          'xarray~=0.19.0',
      ],
      extras_require={
          'dev': [
              'build>=0.5.1',
              'pytest~=6.2.4',
              'twine~=3.4.1',
              'recommonmark~=0.7.1',
              'setuptools~=57.0.0',
              'sphinx~=3.5.1',
              'sphinx-rtd-theme~=0.5.1',
          ]
      },
      entry_points={
          'console_scripts': [
              'analyze_noaa_data = diyepw.scripts.analyze_noaa_data:analyze_noaa_data',
              'create_amy_epw_files = diyepw.scripts.create_amy_epw_files:create_amy_epw_files',
              'create_amy_epw_files_for_years_and_wmos = diyepw.scripts.create_amy_epw_files_for_years_and_wmos:create_amy_epw_files_for_years_and_wmos',
          ],
      },
)



import pandas as pd
import os
from typing import List, Tuple
from .create_amy_epw_file import create_amy_epw_file, _tempdir_amy_epw
from ._logging import _logger

def create_amy_epw_files_for_years_and_wmos(
        years: List[int],
        wmo_indices: List[int],

        *,
        max_records_to_interpolate: int = 6,
        max_records_to_impute: int = 48,
        max_missing_amy_rows: int = 700,
        amy_epw_dir: str = None,
        tmy_epw_dir: str = None,
        amy_dir: str = None,
        amy_files: Tuple[str, str] = None,
        allow_downloads: bool = False
) -> dict:
    """
    Create AMY EPW files for every combination of a set of WMO indices and years.

    Except for "years" and "wmos" being lists rather than single values, all parameters are identical in effect to
    create_amy_epw_file() - see that function's documentation for details.
    :param wmo_indices: A list of the WMO Indices of the weather stations for which the EPW files should be generated.
        Currently only weather stations in the United States are supported.
    :param years: A list of years for which the EPW files should be generated
    :param max_records_to_interpolate: The maximum length of sequence for which linear interpolation will be
        used to replace missing values. See the documentation of _handle_missing_values() below for details.
    :param max_records_to_impute: The maximum length of sequence for which imputation will be used to replace
        missing values. See the documentation of _handle_missing_values() below for details.
    :param max_missing_amy_rows: The maximum total number of missing rows to permit in a year's AMY file.
    :param amy_epw_dir: The directory into which the generated AMY EPW file should be written.
        If not defined, a temporary directory will be created. Note that, in addition to the generated AMY EPW files, an errors.csv file will be created in this
        directory if any errors were encountered, with error messages explaining which year/WMO Index combinations
        failed and why.
    :param tmy_epw_dir: The source directory for TMY EPW files. If a file for the requested WMO Index is
        already present, it will be used. Otherwise a TMY EPW file will be downloaded (see this package's
        get_tmy_epw_file() function for details). If no directory is given, the package's default
        directory (in data/tmy_epw_files/ in the package's directory) will be used, which will allow AMY
        files to be reused for future calls instead of downloading them repeatedly, which is quite time
        consuming.
    :param amy_dir: The source directory for AMY files. If a file for the requested WMO Index and year
        is already present, it will be used. Otherwise a TMY EPW file will be downloaded (see this package's
        get_noaa_isd_lite_file() function for details). If no directory is given, the package's default
        directory (in data/ in the package's directory) will be used, which will allow AMY files to be
        reused for future calls instead of downloading them repeatedly, which is quite time consuming.
    :param amy_files: Instead of specifying amy_dir and allowing this method to try to find the appropriate
        file, you can use this argument to specify the actual files that should be used. There should be
        two files - the first the AMY file for "year", and the second the AMY file for the subsequent year,
        which is required to support shifting the timezone from GMT to the timezone of the observed meteorology.
    :param allow_downloads: If this is set to True, then any missing TMY or AMY files required to generate the
        requested AMY EPW file will be downloaded from publicly available online catalogs. Otherwise, those files
        being missing will result in an error being raised.
    :return: A dictionary of the files generated for each year/wmo combination, in the form {
        <year>: {
            <wmo_index>: [<file_path>, ...],
            ...
        },
        ...
    }
    """
    amy_epw_files = {}

    # Initialize the df to hold information about year/WMO Index combinations for which no EPW could be generated
    errors = pd.DataFrame(columns=['year', 'wmo_index', 'error'])
    errors_path = os.path.join(amy_epw_dir if amy_epw_dir else _tempdir_amy_epw, "errors.csv")

    for year in years:
        for wmo_index in wmo_indices:
            _logger.info(f"Creating AMY EPW for year {year} and WMO {wmo_index}...")

            if not year in amy_epw_files:
                amy_epw_files[year] = {}
            if not wmo_index in amy_epw_files[year]:
                amy_epw_files[year][wmo_index] = []

            try:
                amy_epw_file = create_amy_epw_file(
                    wmo_index,
                    year,
                    max_records_to_interpolate=max_records_to_interpolate,
                    max_records_to_impute=max_records_to_impute,
                    max_missing_amy_rows=max_missing_amy_rows,
                    amy_epw_dir=amy_epw_dir,
                    tmy_epw_dir=tmy_epw_dir,
                    amy_dir=amy_dir,
                    amy_files=amy_files,
                    allow_downloads=allow_downloads
                )
                amy_epw_files[year][wmo_index].append(amy_epw_file)
            except Exception as e:
                errors = errors.append({"year": year, "wmo_index": wmo_index, "error": str(e)}, ignore_index=True)
                print(f'Problem processing year {year} and WMO index {wmo_index}: ' + str(e))

    if not errors.empty:
        _logger.warning(f"AMY EPW files could not be generated for {len(errors)} year/WMO Index combinations - see {errors_path} for more information")
        errors.to_csv(errors_path, mode='w', index=False)

    return amy_epw_files



import unittest
import diyepw
import os

THIS_DIR = os.path.dirname(os.path.abspath(__file__))

class AnalyzeNoaaIsdLiteFilesTest(unittest.TestCase):
    """
    Tests of the behavior of the analyze_noaa_isd_lite_files function
    """

    def test_files(self):
        """
        Confirm the correct behavior of the script when handling our set of test ISD Lite files
        """
        file_paths = map(
            lambda file_name: os.path.join(THIS_DIR, 'files', 'noaa_isd_lite', file_name),
            [
                'max_17_missing_rows',
                'no_missing_rows.bz2',
                'only_one_row',
                'uncompressed_no_missing_rows'
            ]
        )

        analysis = diyepw.analyze_noaa_isd_lite_files(
            file_paths,
            max_missing_rows = 60,
            max_consecutive_missing_rows = 16
        )

        # The two files with no missing rows should be labeled as "good", the one with 17 missing rows should be
        # labeled as "too many consecutive rows missing", and the remaining file, which has only a single row, should
        # be labeled as "too many total rows missing" because that category is given priority over "too many consecutive"
        self.assertEqual(len(analysis['good']), 2)
        for good_res in analysis['good']:
            self.assertEqual(good_res['total_rows_missing'], 0)
            self.assertEqual(good_res['max_consec_rows_missing'], 0)

        self.assertEqual(len(analysis['too_many_total_rows_missing']), 1)
        self.assertEqual(analysis['too_many_total_rows_missing'][0]['total_rows_missing'], 8759)
        self.assertEqual(analysis['too_many_total_rows_missing'][0]['max_consec_rows_missing'], 8759)

        self.assertEqual(len(analysis['too_many_consecutive_rows_missing']), 1)
        self.assertEqual(analysis['too_many_consecutive_rows_missing'][0]['total_rows_missing'], 59)
        self.assertEqual(analysis['too_many_consecutive_rows_missing'][0]['max_consec_rows_missing'], 17)


if __name__ == '__main__': # pragma: no cover
    unittest.main()



import click
import diyepw
from glob import iglob
import os
import pandas as pd


@click.command()
@click.option(
    '--max-missing-rows',
    default=700,
    show_default=True,
    type=int,
    help='ISD files with more than this number of missing rows will be excluded from the output'
)
@click.option(
    '--max-consecutive-missing-rows',
    default=48,
    show_default=True,
    type=int,
    help='ISD files with more than this number of consecutive missing rows will be excluded from the output'
)
@click.option(
    '-o', '--output-path',
    default='.',
    type=click.Path(
        file_okay=False,
        dir_okay=True,
        writable=True,
        resolve_path=True,
    ),
    help="""The path to which output and error files should be written."""
)
@click.argument(
    'input_path',
    default='.',
    type=click.Path(
        file_okay=False,
        dir_okay=True,
        readable=True,
        resolve_path=True,
    ),
)
def analyze_noaa_data(
    max_missing_rows,
    max_consecutive_missing_rows,
    output_path,
    input_path,
):
    """Perform an analysis of a set of NOAA ISA Lite files, determining which are suitable for conversion to
       AMY EPW files. Any ISD Lite files in INPUT_PATH or any of its subdirectories will be processed. The files
       must be named according to the format '<WMO Index>-<WBAN>-<Year>' and must end with '.gz', '.csv', or '.zip'."""

    # Make a directory to store results if it doesn't already exist.
    if not os.path.exists(output_path):
        os.makedirs(output_path)

    # Recursively search for all files under the passed path, excluding directories
    input_files = [file for file in iglob(input_path + '/**/*', recursive=True) if not os.path.isdir(file)]

    try:
        analysis_results = diyepw.analyze_noaa_isd_lite_files(
            input_files,
            max_missing_rows=max_missing_rows,
            max_consecutive_missing_rows=max_consecutive_missing_rows,
        )
    except:
        click.echo("Unable to read input files, aborting...")
        raise click.Abort

    # Write the dataframes to CSVs for the output files.
    num_files_with_too_many_rows_missing = len(analysis_results['too_many_total_rows_missing'])
    if num_files_with_too_many_rows_missing > 0:
        path = os.path.join(output_path, 'missing_total_entries_high.csv')
        path = os.path.abspath(path)  # Change to absolute path for readability
        click.echo(f"""{num_files_with_too_many_rows_missing}
                       records excluded because they were missing more than {max_missing_rows}
                       rows. Information about these files will be written to {path}.""")
        pd.DataFrame(analysis_results['too_many_total_rows_missing']).to_csv(path, index=False)

    num_files_with_too_many_consec_rows_missing = len(analysis_results['too_many_consecutive_rows_missing'])
    if num_files_with_too_many_consec_rows_missing > 0:
        path = os.path.join(output_path, 'missing_consec_entries_high.csv')
        path = os.path.abspath(path)  # Change to absolute path for readability
        click.echo(f"""{num_files_with_too_many_consec_rows_missing}
                       records excluded because they were missing more than {max_consecutive_missing_rows}
                       consecutive rows. Information about these files will be written to {path}.""")
        pd.DataFrame(analysis_results['too_many_consecutive_rows_missing']).to_csv(path, index=False)

    num_good_files = len(analysis_results['good'])
    if num_good_files > 0:
        path = os.path.join(output_path, 'files_to_convert.csv')
        path = os.path.abspath(path)  # Change to absolute path for readability
        click.echo(f"""{num_good_files} records are complete enough to be processed.
                       Information about these files will be written to {path}.""")
        pd.DataFrame(analysis_results['good']).to_csv(path, index=False)

    click.echo('Done! {count} files processed.'.format(count=sum([
        num_good_files,
        num_files_with_too_many_consec_rows_missing,
        num_files_with_too_many_rows_missing
    ])))
